/**
 * Test program.
 */


#include "TextLCD.h"|
#include "mbed.h"
#include "TFT_4DGL.h"
#include "PS2Keyboard.h"
#include "SDFileSystem.h"



//scancodes
#define MK 0x0000
#define BK 0x00F0
#define LCTRL 0x14
#define RCTRL 0x14
#define BKSP 0x66
#define ENTER 0x5A
#define LSHIFT 0x12
#define RSHIFT 0x59
#define E0 0xE0
#define LGUI 0x1F
#define RGUI 0x27
#define RALT 0x11
#define LALT 0x11
#define APPS 0x2F

//screen dimensions (in characters)
#define NUM_COLS 78
#define NUM_ROWS 58




static const unsigned char ps2KeyMap[] = {
//   0    1    2    3    4    5    6    7    8    9    A    B    C    D    E    F
 '\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0', '`','\0', // 00-0F
 '\0','\0','\0','\0','\0', 'Q', '1','\0','\0','\0', 'Z', 'S', 'A', 'W', '2','\0', // 10-1F
 '\0', 'C', 'X', 'D', 'E', '4', '3','\0','\0',' ', 'V', 'F', 'T', 'R', '5','\0', // 20-2F
 '\0', 'N', 'B', 'H', 'G', 'Y', '6','\0','\0', '\0', 'M', 'J', 'U', '7', '8','\0', // 30-3F
 '\0', ',', 'K', 'I', 'O', '0', '9','\0','\0', '.', '/', 'L', ';', 'P', '-','\0', // 40-4F
 '\0','\0', '\'','\0', '[', '=','\0','\0','\0','\0','\n', ']','\0','\\','\0','\0', // 50-5F
 '\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0', // 60-6F
 '\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0', // 70-7F
 '\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0', // 80-8F
 '\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0', // 90-9F
 '\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0', // A0-AF
 '\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0', // B0-BF
 '\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0', // C0-CF
 '\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0', // D0-DF
 '\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0', // E0-EF
 '\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0', // F0-FF
};




SDFileSystem sd(p11, p12, p13, p14, "sd");      //mosi, miso, sclk, cs
TextLCD lcd(p15, p16, p17, p18, p19, p20);      // rs, e, d4-d7
PS2Keyboard ps2kb(p7, p6);                      //clk, data
TFT_4DGL ecran(p9,p10,p8);                      // serial tx, serial rx, reset pin;
Serial scom(USBTX, USBRX);



//prints a cursor to the screen
void printCursor(int col, int row, int color) {
    ecran.line(col*8, row*8, col*8, (row*8)+8, color);
}

//opens a file from a microSD card, displays the contents on the screen, and updates row, column information
void openFile(int *col, int *row, int *buf_col, int *c_count, char *screen) {
    char c;
    FILE *fp = fopen("/sd/mydir/sdtest.txt", "r");
    if (fp == NULL) {
        error("Could not open file for write\n");
    }
    c = fgetc(fp);
    while(!feof(fp)) { 
        screen[*c_count] = c;                                   //store character in screen buffer
        ecran.text_char(screen[*c_count] , *col, *row, WHITE);  //print to VGA screen
        (*col)++;                                               //update 
        buf_col[*row]++;
        (*c_count)++;
        if(c=='\n') {                                           //advance row for new line
            (*row)++;
            (*col)=1;
        }
        if((*col)>NUM_COLS) {                                   //reset col and row for new line
            (*row)++;
            (*col)=1;
        }
        if((*row)>NUM_ROWS) {
            (*row)=1;
            (*col)=1;
        }
        c = fgetc(fp);
    }
    fclose(fp);    
}


//prints cursor position in row, col form on bottom of screen
void printStats(int row, int col, char *s_col, char *s_row) {

     ecran.text_mode(OPAQUE);
     ecran.text_string("row",  55, NUM_ROWS +1, FONT_8X8, DGREY);
     ecran.text_string(s_row,  59, NUM_ROWS +1, FONT_8X8, DGREY);
     ecran.text_string("col",  65, NUM_ROWS +1, FONT_8X8, DGREY);
     ecran.text_string(s_col,  69, NUM_ROWS +1, FONT_8X8, DGREY);
    
     switch (col%10) {
        case 0: s_col[1]='0'; break;
        case 1: s_col[1]='1'; break;
        case 2: s_col[1]='2'; break;
        case 3: s_col[1]='3'; break;
        case 4: s_col[1]='4'; break;
        case 5: s_col[1]='5'; break;
        case 6: s_col[1]='6'; break;
        case 7: s_col[1]='7'; break;
        case 8: s_col[1]='8'; break;
        case 9: s_col[1]='9'; break;
        default: break;
     }
     switch (col/10) {
        case 0: s_col[0]='0'; break;
        case 1: s_col[0]='1'; break;
        case 2: s_col[0]='2'; break;
        case 3: s_col[0]='3'; break;
        case 4: s_col[0]='4'; break;
        case 5: s_col[0]='5'; break;
        case 6: s_col[0]='6'; break;
        case 7: s_col[0]='7'; break;
        case 8: s_col[0]='8'; break;
        case 9: s_col[0]='9'; break;
        default: break;
     }        
     switch (row%10) {
        case 0: s_row[1]='0'; break;
        case 1: s_row[1]='1'; break;
        case 2: s_row[1]='2'; break;
        case 3: s_row[1]='3'; break;
        case 4: s_row[1]='4'; break;
        case 5: s_row[1]='5'; break;
        case 6: s_row[1]='6'; break;
        case 7: s_row[1]='7'; break;
        case 8: s_row[1]='8'; break;
        case 9: s_row[1]='9'; break;
        default: break;
     }
     switch (row/10) {
        case 0: s_row[0]='0'; break;
        case 1: s_row[0]='1'; break;
        case 2: s_row[0]='2'; break;
        case 3: s_row[0]='3'; break;
        case 4: s_row[0]='4'; break;
        case 5: s_row[0]='5'; break;
        case 6: s_row[0]='6'; break;
        case 7: s_row[0]='7'; break;
        case 8: s_row[0]='8'; break;
        case 9: s_row[0]='9'; break;
        default: break;
     }        

     ecran.text_mode(TRANSPARENT);
     ecran.text_string("MEDITor",  20, NUM_ROWS +1, FONT_8X8, GREEN);
     ecran.text_string("row",  55, NUM_ROWS +1, FONT_8X8, GREEN);
     ecran.text_string(s_row,  59, NUM_ROWS +1, FONT_8X8, GREEN);
     ecran.text_string("col",  65, NUM_ROWS +1, FONT_8X8, GREEN);
     ecran.text_string(s_col,  69, NUM_ROWS +1, FONT_8X8, GREEN);
}

int main() {
    int row = 1;                                //cursor row
    int col = 1;                                //cursor col
    int c_count = 0;
    int ctrl_flag = 0, shift_flag = 0, gui_flag = 0, alt_flag = 0, apps_flag = 0, enter_flag=0;//flags for various "control" keys
    int buf_col[NUM_ROWS];                      //last char's col in each row
    char screen[4800];                          //buffer for the screen
    char file_read[4800];                       //output of file read
    unsigned char c;                            //current character being written to screen
    char s_col[2] = {'0','1'};                  //string version of column position
    char s_row[2] = {'0','1'};
    //initialize...   
    for (int i = 0; i < NUM_ROWS; i++) {
        buf_col[i] = 1;
    }

    mkdir("/sd/mydir", 0777);

    ecran.baudrate(115200);
    // added - Set Display to 640 by 480 mode
    ecran.display_control(0x0c, 0x01);
    ecran.background_color(DGREY);
    ecran.pen_size(WIREFRAME);
    ecran.set_font(FONT_8X8);
    ecran.text_mode(TRANSPARENT);
    PS2Keyboard::keyboard_event_t evt_kb;
    openFile(&col, &row, buf_col, &c_count, screen);
    while (1) {
        
        printCursor(col, row, WHITE);//print initial cursor
       
        if (ps2kb.processing(&evt_kb)) {
            if(evt_kb.scancode[0]!=BK) {
                printStats(row, col, s_col, s_row);
            }

            //control code
            switch (evt_kb.scancode[0]) {
                case E0:   //starting scancode for some control codes
                    switch (evt_kb.scancode[1]) {
                        case RCTRL:  //rctrl was pressed
                            ctrl_flag =1;
                            break;
                        case LGUI:// left "windows" button was preswsed
                            gui_flag =1;
                            break;
                        case RGUI:  //rctrl was pressed
                            gui_flag =1;
                            break;
                        case RALT:// left "windows" button was preswsed
                            alt_flag =1;
                            break;
                        case APPS:  //apps was pressed
                            apps_flag =1;
                            break;
                        default:
                            break;
                    }
                case LCTRL:
                    ctrl_flag = 1;
                    break;
                case LSHIFT:
                case RSHIFT:
                    shift_flag =1;
                    break;
                case BK:
                    shift_flag=0;
                    break;
                case ENTER:
                    // buf_col[row] = col;
                    enter_flag =1;
                    break;
                case BKSP:
                    if (c_count>0) {//check if there are any characters that can be deleted
                        ecran.text_mode(OPAQUE);
                        printCursor(col, row, DGREY);
                        if ((col-1)==0) { //char to be deleted not on the current row
                            buf_col[row]=1;
                            row--;
                            col = buf_col[row];
                        }
                        ecran.text_char(screen[c_count-1] , col-1, row, DGREY);//printing lowercase
                        screen[c_count-1] = '\0';
                        c_count--;
                        ecran.text_mode(TRANSPARENT);
                        col--;
                        buf_col[row]--;
                        if (col==0) {
                            buf_col[row]=1;
                            row--;
                            col = buf_col[row];
                        }
                        printCursor(col, row, WHITE);
                    }
                    break;
                default:
                    break;
            }
           
            c = ps2KeyMap[evt_kb.scancode[0]];              //get keyboard input and map to the right key

            if (ctrl_flag) {
                if (c == 'S') {
                    ecran.text_string("Saving File...",  1, NUM_ROWS +1, FONT_8X8, WHITE);
                    FILE *fp = fopen("/sd/mydir/sdtest.txt", "w");
                    if (fp == NULL) {
                        error("Could not open file for write\n");
                    }
                    for (int i = 0; i < c_count; i++) {
                        fprintf(fp, "%c", screen[i]);
                    }
                    fclose(fp);
                    ctrl_flag = 0;

                    //test file read -- DEBUG
                    fp = fopen("/sd/mydir/sdtest.txt", "r");
                    fread(file_read,1,c_count,fp);

                    scom.printf("File read...\n");
                    for (int i = 0; i < c_count;i++) {
                        scom.printf("%c",file_read[i]);
                    }
                    fclose(fp);
                    ecran.text_mode(OPAQUE);
                    ecran.text_string("Saving File...",  1, NUM_ROWS +1, FONT_8X8, DGREY);
                    ecran.text_mode(TRANSPARENT);
                }
            }

            else if (c) {
                screen[c_count] = c;
                c_count++;
                //ecran.rectangle((col-1)*8, (row)*8 +1 , (col-1)*8 +1, (row+1)*8, DGREY);
                if (c >= 'A' && c <= 'Z' && !shift_flag) {
                    screen[c_count-1] = c+0x20;
                    printCursor(col, row, DGREY);
                    ecran.text_char( (c+0x20), col, row, WHITE);//printing lowercase
                    col++;
                    buf_col[row]++;
                    if (col < NUM_COLS) {
                        printCursor(col, row, WHITE);
                    }
                } else {
                    screen[c_count-1] = c;
                    printCursor(col, row, DGREY);
                    ecran.text_char( c, col, row, WHITE);//printing lowercase
                    buf_col[row]++;
                    col++;
                    if (enter_flag) {
                        row++;
                        col = 1;
                        enter_flag=0;
                    }
                    if (col < NUM_COLS) {
                        printCursor(col, row, WHITE);
                    }
                }

            }
            //wrap row and col
            if (col > NUM_COLS) {
                row++;
                col = 1;
            }
            if (row > NUM_ROWS) {
                row = 1;
                col = 1;
            }
        }          //if keyboard event
    }              //while
}                  //main
